'use strict';

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

//=============================================================================
// Rem_Dictionary.js
//=============================================================================

/*:
* @author Kino
* @plugindesc A plugin that allows you to have a dictionary in your game
*
*
* @help
* Version 1.01
* ChangeLog:
*  * Supports Text Codes
*  * Word Wrapping
*  * Icon support
*  * Category Support
*  * WhiteList Support
*    * Add/Remove categories and their dictionary files 
//=============================================================================
// Introduction
//=============================================================================
* This is a plugin that works with the base Rem_Heart plugin. This plugin taps
* into the dictionary folder created by the plugin, giving the dev a dictionary
* with all the words in their dictionary folder.
*
* To effectively use this plugin, you will have to have Rem_Heart.js above this
* plugin in the file Plugin hierarchy. Afterwards, you can use it as necessary.
*
//=============================================================================
//	Script Calls
//=============================================================================
* You can start the scene by calling SceneManager.gotoDictionaryScene();
*
* For the advanced scene: SceneManager.gotoDictionarySceneAdv();
* This scene supports categories using a new category.json file for creating
* categories in the dictionary file.
* Pressing cancel or right clicking closes the scene.
*
*
* --- White List Script Calls ---
* The White List script calls allow you to white list certain dictionary
* elements and definitions from bein displayed in the Advanced Dictionary
* Scene.
*
* RemDictionary.whiteListCategory(categoryName);
*
* Example: RemDictionary.whiteListCategory("People");
* This would remove the people category from the advanced scene.
*
* RemDictionary.unwhiteListCategory(categoryName);
*
* Example: RemDictionary.unwhiteListCategory("People")
* This would add the people category back to the advanced scene.
* 
* RemDictionary.whiteListContent(contentName);
*
* Example:RemDictionary.whiteListContent("Kino");
* This would remove Kino from the dictionary scene under whatever
* category he's under.
*
* RemDictionary.unwhiteListContent(contentName)
*
* Example: RemDictionary.unwhiteListContent("Kino")
* This would add Kino from the dictionary back to his related
* category.
*
* --- List Versions ---
* RemDictionary.whiteListCategories(list)
* 
* Example: RemDictionary.whiteListCategories(["People", "Location"])
* This would remove both the people and location category from the
* advanced scene in a single script call.
*
* RemDictionary.whiteListContents(list)
*
* Example: RemDictionary.whiteListContents(["Kino", "Anime"])
* This would remove Kino and Anime both from the respective categories.
*
* RemDictionary.unwhiteListCategories(list)
*
* Example: RemDictionary.unwhiteListCategories(["People", "Location"])
* This would add People and Location back to the category selection area.
* 
* RemDictionary.unwhiteListContents(list)
*
* Example: RemDictionary.whiteListContents(["Kino", "Anime"])
* This would add Kino and Anime back to their respective categories.
*
//=============================================================================
//  Contact Information
//=============================================================================
*
* Contact me via twitter: EISKino, or on the rpg maker forums.
* Username on forums: Kino.
*
* Forum Link: http://forums.rpgmakerweb.com/index.php?/profile/75879-kino/
* Website Link: http://endlessillusoft.com/
* Twitter Link: https://twitter.com/EISKino
* Patreon Link: https://www.patreon.com/EISKino
*
* Hope this plugin helps, and enjoy!
* --Kino
*/

var RemDictionary = RemDictionary || {};
(function ($) {

  'use strict';

  var params = PluginManager.parameters('Rem_Dictionary');
  function Setup($) {

    //=============================================================================
    // Scene_RemDictionary
    //=============================================================================

    var Scene_RemDictionary = function (_Scene_MenuBase) {
      _inherits(Scene_RemDictionary, _Scene_MenuBase);

      function Scene_RemDictionary() {
        _classCallCheck(this, Scene_RemDictionary);

        return _possibleConstructorReturn(this, (Scene_RemDictionary.__proto__ || Object.getPrototypeOf(Scene_RemDictionary)).call(this));
      }

      _createClass(Scene_RemDictionary, [{
        key: 'create',
        value: function create() {
          Scene_MenuBase.prototype.create.call(this);
          this.createAllWindows();
        }
      }, {
        key: 'createAllWindows',
        value: function createAllWindows() {
          this.createWindowWordList();
          this.createWindowWordTitle();
          this.createWindowWord();
        }
      }, {
        key: 'createWindowWordList',
        value: function createWindowWordList() {
          this._wordListWindow = new Window_WordList(0, 0, 250, Graphics.height);
          this.addChild(this._wordListWindow);
        }
      }, {
        key: 'createWindowWordTitle',
        value: function createWindowWordTitle() {
          this._wordTitleWindow = new Window_WordTitle(250, 0, Graphics.width - 250, 75);
          this.addChild(this._wordTitleWindow);
        }
      }, {
        key: 'createWindowWord',
        value: function createWindowWord() {
          this._wordWindow = new Window_Word(250, 75, Graphics.width - 250, Graphics.height - 75);
          this.addChild(this._wordWindow);
        }
      }, {
        key: 'update',
        value: function update() {
          Scene_MenuBase.prototype.update.call(this);
          this.getCurrentWordForTitle();
          this.getCurrentWordInfoForText();
          if (Input.isTriggered('cancel') || TouchInput.isCancelled()) this.popScene();
        }
      }, {
        key: 'getCurrentWordForTitle',
        value: function getCurrentWordForTitle() {
          this._wordTitleWindow._word = this._wordListWindow.getCurrentWord();
        }
      }, {
        key: 'getCurrentWordInfoForText',
        value: function getCurrentWordInfoForText() {
          this._wordWindow.setDefinition(this._wordListWindow.getCurrentWordInformation());
        }
      }]);

      return Scene_RemDictionary;
    }(Scene_MenuBase);

    //=============================================================================
    //  Scene_RemDictionaryAdv
    //=============================================================================    


    var Scene_RemDictionaryAdv = function (_Scene_RemDictionary) {
      _inherits(Scene_RemDictionaryAdv, _Scene_RemDictionary);

      function Scene_RemDictionaryAdv() {
        _classCallCheck(this, Scene_RemDictionaryAdv);

        var _this2 = _possibleConstructorReturn(this, (Scene_RemDictionaryAdv.__proto__ || Object.getPrototypeOf(Scene_RemDictionaryAdv)).call(this));

        RemDictionary.createWhiteListCompendium();
        return _this2;
      }

      _createClass(Scene_RemDictionaryAdv, [{
        key: 'createAllWindows',
        value: function createAllWindows() {
          this.createWindowWordAdv();
          this.createWindowWordTitle();
          this.createWindowWord();
        }
      }, {
        key: 'createWindowWordAdv',
        value: function createWindowWordAdv() {
          this._wordListWindow = new Window_WordAdv(0, 0, 250, Graphics.height);
          this._wordListWindow.setHandler('ok', this.processListOk.bind(this));
          this._wordListWindow.setHandler('cancel', this.processListCancel.bind(this));
          this.addWindow(this._wordListWindow);
        }
      }, {
        key: 'update',
        value: function update() {
          Scene_MenuBase.prototype.update.call(this);
          this.processWordInformation();
          this.processSceneExit();
        }
      }, {
        key: 'processWordInformation',
        value: function processWordInformation() {
          this.getCurrentWordForTitle();
          this.getCurrentWordInfoForText();
        }
      }, {
        key: 'processSceneExit',
        value: function processSceneExit() {
          if (Input.isTriggered('cancel') || TouchInput.isCancelled()) this.popScene();
        }
      }, {
        key: 'processListOk',
        value: function processListOk() {
          if (this._wordListWindow.mode === 'category') {
            this._wordListWindow.selectCategory();
            console.log("ran");
            console.log(this._wordListWindow.mode);
          }
        }
      }, {
        key: 'processListCancel',
        value: function processListCancel() {
          if (this._wordListWindow.mode === 'word') {
            this._wordListWindow.deselectCategory();
            console.log(this._wordListWindow.mode);
          } else {
            this.popScene();
          }
        }
      }]);

      return Scene_RemDictionaryAdv;
    }(Scene_RemDictionary);
    //=============================================================================
    // Window_WordList
    //=============================================================================

    var Window_WordList = function (_Window_Selectable) {
      _inherits(Window_WordList, _Window_Selectable);

      function Window_WordList(x, y, width, height) {
        _classCallCheck(this, Window_WordList);

        return _possibleConstructorReturn(this, (Window_WordList.__proto__ || Object.getPrototypeOf(Window_WordList)).call(this, x, y, width, height));
      }

      _createClass(Window_WordList, [{
        key: 'initialize',
        value: function initialize(x, y, width, height) {
          _get(Window_WordList.prototype.__proto__ || Object.getPrototypeOf(Window_WordList.prototype), 'initialize', this).call(this, x, y, width, height);
          this._dictionary = RemIn.getCompendium().dictionary;
          this._list = [];
          this.insertDictionaryDefinitions();
          this.select(0);
          this.activate();
        }
      }, {
        key: 'insertDictionaryDefinitions',
        value: function insertDictionaryDefinitions() {
          for (var word in this._dictionary) {
            if (!/category/i.test(this._dictionary[word].originalFileName)) {
              console.log(this._dictionary[word]);
              this._list.push({ name: this._dictionary[word].originalFileName, definition: this._dictionary[word].text });
            }
          }
        }
      }, {
        key: 'maxItems',
        value: function maxItems() {
          var dictionaryLength = Object.keys(RemIn.getCompendium().dictionary).length - 1;
          return dictionaryLength;
        }
      }, {
        key: 'update',
        value: function update() {
          _get(Window_WordList.prototype.__proto__ || Object.getPrototypeOf(Window_WordList.prototype), 'update', this).call(this);
          this.refresh();
        }
      }, {
        key: 'drawItem',
        value: function drawItem(index) {
          var rect = this.itemRectForText(index);
          var item = this._list[index];
          if (item !== undefined) this.drawText(item.name.capitalize(), rect.x, rect.y, rect.width);
        }
      }, {
        key: 'getCurrentWord',
        value: function getCurrentWord() {
          if (this._list[this.index()] !== undefined) return this._list[this.index()].name;
        }
      }, {
        key: 'getCurrentWordInformation',
        value: function getCurrentWordInformation() {
          if (this._list[this.index()] !== undefined) return this._list[this.index()].definition;
        }
      }]);

      return Window_WordList;
    }(Window_Selectable);

    //=============================================================================
    //  Window_WordAdv
    //=============================================================================    


    var Window_WordAdv = function (_Window_Selectable2) {
      _inherits(Window_WordAdv, _Window_Selectable2);

      function Window_WordAdv(x, y, width, height) {
        _classCallCheck(this, Window_WordAdv);

        return _possibleConstructorReturn(this, (Window_WordAdv.__proto__ || Object.getPrototypeOf(Window_WordAdv)).call(this, x, y, width, height));
      }

      _createClass(Window_WordAdv, [{
        key: 'initialize',
        value: function initialize(x, y, width, height) {
          this._list = [];
          this._dictionary = RemDictionary.getCompendium();
          this._mode = 'category';
          this._prevIndex = 0;
          this.insertDictionaryDefinitions();
          _get(Window_WordAdv.prototype.__proto__ || Object.getPrototypeOf(Window_WordAdv.prototype), 'initialize', this).call(this, x, y, width, height);
          this.select(0);
          this.activate();
        }
      }, {
        key: 'insertDictionaryDefinitions',
        value: function insertDictionaryDefinitions() {
          this._list = RemDictionary.getwhiteListCompendium().categories;
        }
      }, {
        key: 'maxItems',
        value: function maxItems() {
          return this._list.length;
        }
      }, {
        key: 'update',
        value: function update() {
          _get(Window_WordAdv.prototype.__proto__ || Object.getPrototypeOf(Window_WordAdv.prototype), 'update', this).call(this);
          this.refresh();
        }
      }, {
        key: 'drawItem',
        value: function drawItem(index) {
          var word = '';
          var rect = this.itemRectForText(index);
          if (this._mode === 'category') {
            if (this._list[index] !== undefined) word = this._list[index].category;
            this.drawText(word, rect.x, rect.y, rect.width);
          }
          if (this._mode === 'word') {
            if (this._list[index] !== undefined) word = this._list[index].originalFileName;
            this.drawText(word, rect.x, rect.y, rect.width);
          }
        }
      }, {
        key: 'selectCategory',
        value: function selectCategory() {
          this.setMode('word');
          this._list = this._list[this.index()].list;
          this._prevIndex = this.index();
          this.select(0);
        }
      }, {
        key: 'deselectCategory',
        value: function deselectCategory() {
          this.setMode('category');
          this.insertDictionaryDefinitions();
          this.select(this._prevIndex);
        }
      }, {
        key: 'setMode',
        value: function setMode(mode) {
          this._mode = mode;
        }
      }, {
        key: 'getCurrentWord',
        value: function getCurrentWord() {
          var word = '';
          if (this._mode === 'category') {
            if (this._list[this.index()] !== undefined) word = this._list[this.index()].category;
          }
          if (this._mode === 'word') {
            if (this._list[this.index()] !== undefined) word = this._list[this.index()].originalFileName;
          }
          return word;
        }
      }, {
        key: 'getCurrentWordInformation',
        value: function getCurrentWordInformation() {
          var info = '';
          if (this._mode === 'category') {
            if (this._list[this.index()] !== undefined) info = this._list[this.index()].description;
          }
          if (this._mode === 'word') {
            if (this._list[this.index()] !== undefined) info = this._list[this.index()].text;
          }
          return info;
        }
      }, {
        key: 'processOk',
        value: function processOk() {
          if (this.isCurrentItemEnabled()) {
            this.playOkSound();
            this.updateInputData();
            this.callOkHandler();
          } else {
            this.playBuzzerSound();
          }
          this.refresh();
        }
      }, {
        key: 'processCancel',
        value: function processCancel() {
          SoundManager.playCancel();
          this.updateInputData();
          this.callCancelHandler();
        }
      }, {
        key: 'mode',
        get: function get() {
          return this._mode;
        }
      }]);

      return Window_WordAdv;
    }(Window_Selectable);
    //=============================================================================
    // Window_WordTitle                                                             
    //=============================================================================

    var Window_WordTitle = function (_Window_Base) {
      _inherits(Window_WordTitle, _Window_Base);

      function Window_WordTitle(x, y, width, height) {
        _classCallCheck(this, Window_WordTitle);

        return _possibleConstructorReturn(this, (Window_WordTitle.__proto__ || Object.getPrototypeOf(Window_WordTitle)).call(this, x, y, width, height));
      }

      _createClass(Window_WordTitle, [{
        key: 'initialize',
        value: function initialize(x, y, width, height) {
          Window_Base.prototype.initialize.call(this, x, y, width, height);
          this._word = "";
        }
      }, {
        key: 'update',
        value: function update() {
          Window_Base.prototype.update.call(this);
          this.refresh();
        }
      }, {
        key: 'refresh',
        value: function refresh() {
          if (this.contents) {
            this.contents.clear();
            this.drawTitle();
          }
        }
      }, {
        key: 'drawTitle',
        value: function drawTitle(index) {
          var word = this._word;
          var midPoint = this.contentsWidth() / 2 - this.textWidth(word) / 2;
          this.drawText(word.capitalize(), midPoint, 0, 300);
        }
      }]);

      return Window_WordTitle;
    }(Window_Base);
    //=============================================================================
    // Window_Word
    //=============================================================================

    var Window_Word = function (_Window_Base2) {
      _inherits(Window_Word, _Window_Base2);

      function Window_Word(x, y, width, height) {
        _classCallCheck(this, Window_Word);

        return _possibleConstructorReturn(this, (Window_Word.__proto__ || Object.getPrototypeOf(Window_Word)).call(this, x, y, width, height));
      }

      _createClass(Window_Word, [{
        key: 'initialize',
        value: function initialize(x, y, width, height) {
          this.currentWord = [];
          this._definition;
          Window_Base.prototype.initialize.call(this, x, y, width, height);
        }
      }, {
        key: 'update',
        value: function update() {
          Window_Base.prototype.update.call(this);
          this.refresh();
        }
      }, {
        key: 'refresh',
        value: function refresh() {
          if (this.contents) {
            this.contents.clear();
            this.drawDefinition();
          }
        }
      }, {
        key: 'drawDefinition',
        value: function drawDefinition(index) {
          var definition = this._definition;
          this.drawTextEx(definition, 0, 0);
        }
      }, {
        key: 'drawTextEx',
        value: function drawTextEx(text, x, y) {
          if (text) {
            var textState = { index: 0, x: x, y: y, left: x };
            textState.text = this.convertEscapeCharacters(text);
            textState.height = this.calcTextHeight(textState, false);
            this.resetFontSettings();
            this.wordShiftAmount = textState.left;
            this.currentWord.length = 0;
            while (textState.index < textState.text.length) {
              this.processCharacter(textState);
            }
            return textState.x - x;
          } else {
            return 0;
          }
        }
      }, {
        key: 'processNormalCharacter',
        value: function processNormalCharacter(textState) {
          var num = textState.index++;
          var char = textState.text[num];
          var length = textState.text.length;
          var charWidth = this.textWidth(char);

          if (/\S/ig.test(char)) {
            this.updateWordArray(char, textState);
          }

          if ((/\s/ig.test(char) || /[\W]/ig.test(char) || length === num + 1) && this.currentWord.length > 0) {
            var word = this.createWord(this.currentWord);
            var firstLetter = this.currentWord[0];
            var wordWidth = this.textWidth(word);

            if (firstLetter.x + wordWidth > this.contentsWidth()) {
              this.updateTextStateAndShiftAmount(firstLetter, wordWidth, textState);
            }

            word = !/[\W]/ig.test(char) && length === num + 1 ? word : word;
            this.contents.drawText(word, firstLetter.x, firstLetter.y, wordWidth * 2, firstLetter.align);
            this.currentWord.length = 0;
          }

          textState.x += charWidth;
        }
      }, {
        key: 'updateWordArray',
        value: function updateWordArray(char, textState) {
          var x = this.currentWord.length === 0 ? this.wordShiftAmount + textState.x : textState.x;
          var charWidth = this.textWidth(char);
          this.currentWord.push({ char: char, x: x, y: textState.y, width: charWidth * 2, align: textState.height });
        }
      }, {
        key: 'createWord',
        value: function createWord(letterArray) {
          var word = letterArray.map(function (element) {
            return element.char;
          });
          return word.join("");
        }
      }, {
        key: 'updateTextStateAndShiftAmount',
        value: function updateTextStateAndShiftAmount(firstLetter, wordWidth, textState) {
          textState.x = textState.left;
          textState.y += textState.height;
          firstLetter.x = textState.x;
          firstLetter.y = textState.y;
          firstLetter.align = textState.height;
          this.wordShiftAmount = firstLetter.x + wordWidth;
        }
      }, {
        key: 'setDefinition',
        value: function setDefinition(definition) {
          this._definition = definition;
        }
      }]);

      return Window_Word;
    }(Window_Base);

    //=============================================================================
    //  RemDictionary
    //=============================================================================    


    var RemDictionary = function () {
      function RemDictionary() {
        _classCallCheck(this, RemDictionary);
      }

      _createClass(RemDictionary, null, [{
        key: 'initialize',
        value: function initialize() {
          this._whiteList = { categories: [], content: [] };
          this._compendium = RemIn.getCompendium().dictionary;
          this._categories = this.getCategoryInformation();
          this._categorizedCompendium = this.categorizeCompendium();
          this._whiteListCompendium = JsonEx.makeDeepCopy(this._categorizedCompendium);
        }
      }, {
        key: 'getCategoryInformation',
        value: function getCategoryInformation() {
          var path = Utils.isNwjs() ? "Rem/dictionary/" : "/Rem/dictionary";
          var categoryData = JSON.parse(RemIn.readCustomFile(path, "category.json", 'utf8'));
          return categoryData;
        }
      }, {
        key: 'categorizeCompendium',
        value: function categorizeCompendium() {
          var compendium = JsonEx.makeDeepCopy(this._compendium);
          for (var word in compendium) {
            this.updateCategory(compendium[word]);
          }
          return JsonEx.makeDeepCopy(this._categories);
        }
      }, {
        key: 'updateCategory',
        value: function updateCategory(wordData) {
          if (this._categories === null && !Utils.isNwjs()) this._categories = JsonEx.parse(localStorage.getItem("category.json.json"));
          var categories = this._categories.categories;
          if (categories !== null) {
            categories.forEach(function (category) {
              category.list.forEach(function (fileName, index) {
                if (wordData.originalFileName === fileName) {
                  category.list[index] = wordData;
                }
              });
            });
          }
        }
      }, {
        key: 'whiteListCategory',
        value: function whiteListCategory(categoryName) {
          if (!this._whiteList.categories.some(function (category) {
            return new RegExp('' + categoryName, "i").test(category);
          })) {
            this._whiteList.categories.push(categoryName);
          }
        }
      }, {
        key: 'whiteListContent',
        value: function whiteListContent(contentName) {
          if (!this._whiteList.content.some(function (content) {
            return new RegExp('' + contentName, "i").test(content);
          })) {
            this._whiteList.content.push(contentName);
          }
        }
      }, {
        key: 'unwhiteListContent',
        value: function unwhiteListContent(contentName) {
          var _this7 = this;

          this._whiteList.content.forEach(function (content, index) {
            if (new RegExp('' + contentName, "ig").test(content)) {
              _this7._whiteList.content.splice(index, 1);
            }
          });
        }
      }, {
        key: 'unwhiteListCategory',
        value: function unwhiteListCategory(categoryName) {
          var _this8 = this;

          this._whiteList.categories.forEach(function (category, index) {
            if (new RegExp('' + categoryName, "ig").test(category)) {
              _this8._whiteList.categories.splice(index, 1);
            }
          });
        }
      }, {
        key: 'createWhiteListCompendium',
        value: function createWhiteListCompendium() {
          this._whiteListCompendium = JsonEx.makeDeepCopy(this._categorizedCompendium);
          this.createWhiteListCompendiumCategories();
          this.createWhiteListCompendiumContent();
          console.log(this._whiteListCompendium);
        }
      }, {
        key: 'createWhiteListCompendiumCategories',
        value: function createWhiteListCompendiumCategories() {
          var _this9 = this;

          this._whiteList.categories.forEach(function (category) {
            _this9._whiteListCompendium.categories.forEach(function (compendiumCategory, index) {
              if (new RegExp('' + category, "i").test(compendiumCategory.category) === true) {
                _this9._whiteListCompendium.categories.splice(index, 1);
              }
            });
          });
        }
      }, {
        key: 'createWhiteListCompendiumContent',
        value: function createWhiteListCompendiumContent() {
          var _this10 = this;

          this._whiteList.content.forEach(function (contentName) {
            _this10._whiteListCompendium.categories.forEach(function (compendiumCategory) {
              compendiumCategory.list.forEach(function (contentItem, index) {
                if (new RegExp('' + contentName, "i").test(contentItem.originalFileName)) {
                  compendiumCategory.list.splice(index, 1);
                }
              });
            });
          });
        }
      }, {
        key: 'setwhiteList',
        value: function setwhiteList(whitelist) {
          this._whiteList = whitelist;
        }
      }, {
        key: 'getwhiteList',
        value: function getwhiteList() {
          return this._whiteList;
        }
      }, {
        key: 'getCategories',
        value: function getCategories() {
          return this._categories.map(function (category) {
            return category.category;
          });
        }
      }, {
        key: 'getCompendium',
        value: function getCompendium() {
          return this._compendium;
        }
      }, {
        key: 'getCategorizedCompendium',
        value: function getCategorizedCompendium() {
          return this._categorizedCompendium;
        }
      }, {
        key: 'getwhiteListCompendium',
        value: function getwhiteListCompendium() {
          return this._whiteListCompendium;
        }
      }]);

      return RemDictionary;
    }();

    //=============================================================================
    //  Startup
    //=============================================================================    


    setTimeout(function () {
      RemDictionary.initialize();
    }, 5000);
    //=============================================================================
    // String
    //=============================================================================
    String.prototype.capitalize = function () {
      return this.charAt(0).toUpperCase() + this.slice(1);
    };
    //=============================================================================
    //  DataManager
    //=============================================================================
    var _DataManager_makeSaveContents = DataManager.makeSaveContents;
    DataManager.makeSaveContents = function () {
      var contents = {};
      contents = _DataManager_makeSaveContents.call(this);
      contents.RemWhiteList = RemDictionary.getwhiteList();
      return contents;
    };

    var _DataManager_extractSaveContents = DataManager.extractSaveContents;
    DataManager.extractSaveContents = function (contents) {
      _DataManager_extractSaveContents.call(this, contents);
      RemDictionary.setwhiteList(contents.RemWhiteList);
    };
    //=============================================================================
    // SceneManager
    //=============================================================================
    SceneManager.gotoDictionaryScene = function () {
      SceneManager.push(Scene_RemDictionary);
    };

    SceneManager.gotoDictionarySceneAdv = function () {
      SceneManager.push(Scene_RemDictionaryAdv);
    };
    //=============================================================================
    // Exports
    //=============================================================================
    window.Scene_RemDictionary = Scene_RemDictionary;
    window.Scene_RemDictionaryAdv = Scene_RemDictionaryAdv;
    //=============================================================================
    //  API
    //=============================================================================
    Object.assign($, {
      whiteListCategories: function whiteListCategories(list) {
        list.forEach(function (category) {
          RemDictionary.whiteListCategory(category);
        });
      },
      whiteListCategory: function whiteListCategory(categoryName) {
        RemDictionary.whiteListCategory(categoryName);
      },
      whiteListContents: function whiteListContents(list) {
        list.forEach(function (contentName) {
          RemDictionary.whiteListContent(contentName);
        });
      },
      whiteListContent: function whiteListContent(contentName) {
        RemDictionary.whiteListContent(contentName);
      },
      unwhiteListCategory: function unwhiteListCategory(categoryName) {
        RemDictionary.unwhiteListCategory(categoryName);
      },
      unwhiteListContent: function unwhiteListContent(contentName) {
        RemDictionary.unwhiteListContent(contentName);
      },
      unwhiteListCategories: function unwhiteListCategories(list) {
        list.forEach(function (category) {
          RemDictionary.unwhiteListCategory(category);
        });
      },
      unwhiteListContents: function unwhiteListContents(list) {
        list.forEach(function (contentName) {
          RemDictionary.unwhiteListContent(contentName);
        });
      }
    });
  }

  Setup($);
})(RemDictionary);