//=============================================================================
// SOUL_MV Item Book EX.js by Soulpour777 - soulxregalia.wordpress.com
//=============================================================================

/*:
 * @plugindesc v1.0 Displays a dictionary of items with details and images.
 * @author Soulpour777 - soulxregalia.wordpress.com
 *
 * @param -- MAIN --
 * 
 * @param Show on Menu
 * @desc Would you like the Item Book Ex to be in your Menu?
 * @default true
 * 
 * @param Background
 * @desc Your default Item Book Ex background. Changable in game as well.
 * @default CrossedSwords
 * 
 * @param Scroll X
 * @desc Background Horizontal Scroll Speed
 * @default 0
 * 
 * @param Scroll Y
 * @desc Background Vertical Scroll Speed
 * @default 0
 * 
 * @param Scroll Y
 * @desc Background Vertical Scroll Speed
 * @default 0
 *
 * @param Item Picture X
 * @desc The x axis of the item picture that represents your item.
 * @default 550
 * 
 * @param Item Picture Y
 * @desc The y axis of the item picture that represents your item.
 * @default 10
 * 
 * @param Item Picture Opacity
 * @desc The opacity of the item picture that represents your item.
 * @default 255

 * @param -- WINDOW TITLE --
 * 
 * @param WT_Width
 * @desc The title window width.
 * @default 816
 * 
 * @param WT_Height
 * @desc The title window height.
 * @default 75
 * 
 * @param WT_X
 * @desc The title window x axis.
 * @default 0
 * 
 * @param WT_Y
 * @desc The title window y axis.
 * @default 0
 * 
 * @param WT_TextX
 * @desc The title window text horizontal margin.
 * @default 290
 * 
 * @param WT_TextY
 * @desc The title window text vertical margin. (0 by default so it would show on the top.)
 * @default 0
 * 
 * @param WT_OPACITY
 * @desc The title window opacity.
 * @default 255
 *
 * @param -- ITEM LIST --
 * 
 * @param LIST_Width
 * @desc The item list window width.
 * @default 816
 * 
 * @param LIST_Height
 * @desc The item list window height.
 * @default 146
 * 
 * @param LIST_X
 * @desc The item list window x axis.
 * @default 0
 * 
 * @param LIST_Y
 * @desc The item list window y axis.
 * @default 75
 * 
 * @param LIST_OPACITY
 * @desc The item list window opacity.
 * @default 255
 *
 * @param -- WINDOW DETAILS --
 * 
 * @param D_Width
 * @desc The details window width.
 * @default 816
 * 
 * @param D_Height
 * @desc The details window height.
 * @default 403
 * 
 * @param D_X
 * @desc The details list window x axis.
 * @default 0
 * 
 * @param D_Y
 * @desc The details list window y axis.
 * @default 220
 * 
 * @param D_OPACITY
 * @desc The details list window opacity.
 * @default 255
 *
 * @param -- TEXTS --
 *
 * @param Menu Command Name
 * @desc The command name of the Item Book Ex from the Menu Command.
 * @default Item Regalia
 *
 * @param Window Title
 * @desc The window title name for the Item Book EX.
 * @default Item Regalia
 *
 * @param Unidentified
 * @desc The text used when an item is still undefined / not defined.
 * @default ??????
 *
 * @param Item Price
 * @desc The text used for the item's price label.
 * @default Price
 *
 * @param Equipment
 * @desc The text used for the equipment label.
 * @default Equip
 *
 * @param Item Type
 * @desc The text used for the item type label.
 * @default Type
 *
 * @help
 *
 * Plugin Commands:
 *   ItemBookEX open            # Opens the ItemBookEx scene.
 *   ItemBookEX add item x      # Where x is the id of the item that you want to add in the book.
 *   ItemBookEX add weapon x    # Where x is the id of the weapon that you want to add in the book.
 *   ItemBookEX add armor x     # Where x is the id of the armor that you want to add in the book.
 *   ItemBookEX remove weapon x # Where x is the id of the weapon that you want to remove in the book.
 *   ItemBookEX remove armor x  # Where x is the id of the armor that you want to remove in the book.
 *   ItemBookEX remove item x   # Where x is the id of the item that you want to remove in the book.+
 *   ItemBookEX background x    # Where x is the name of the background you want to use. (Single Word only)
 *   ItemBookEX show_on_menu x  # Where x is either true or false. (Shows Item Book on Menu or not)
 *   ItemBookEX complete        # Completes the whole book.
 *   ItemBookEX clear           # Clears the contents of the whole book.

 *
 * Note Tags:
 *   <item book exclude>     
 *   This will exclude the item in the book when placed on items / weapons
 *   and armor's note box.
 *
 * Note:
 *   Place all needed images under the img / ItemBookEx folder.    
 *   The picture that represents the item should have the same name as that item.
 *
 *   Press F8 to see what errors you have about the plugin and see what the error is. If
 *   it has to do with the images not existent in your folder, it means you have to place
 *   the picture there.
 */


(function() {
    var SOUL_MV = SOUL_MV || {};
    SOUL_MV.ItemBookEx = SOUL_MV.ItemBookEx || {};

    SOUL_MV.ItemBookEx.show = PluginManager.parameters('SOUL_MV Item Book EX')['Show on Menu'] === 'true' ? true : false;
    SOUL_MV.ItemBookEx.menuName = PluginManager.parameters('SOUL_MV Item Book EX')['Menu Command Name'] || 'Item Regalia';

    SOUL_MV.ItemBookEx.pictureX = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['Item Picture X'] || 550);
    SOUL_MV.ItemBookEx.pictureY = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['Item Picture Y'] || 10);
    SOUL_MV.ItemBookEx.pictureOpacity = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['Item Picture Opacity'] || 255);

    SOUL_MV.ItemBookEx.background = PluginManager.parameters('SOUL_MV Item Book EX')['Background'] || 'CrossedSwords';
    SOUL_MV.ItemBookEx.backgroundScrollX = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['Scroll X'] || 0);
    SOUL_MV.ItemBookEx.backgroundScrollY = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['Scroll Y'] || 0);

    SOUL_MV.ItemBookEx.windowTitleX = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_X'] || 0);
    SOUL_MV.ItemBookEx.windowTitleY = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_Y'] || 0);
    SOUL_MV.ItemBookEx.windowTitleTextX = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_TextX'] || 290);
    SOUL_MV.ItemBookEx.windowTitleTextY = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_TextY'] || 0);

    SOUL_MV.ItemBookEx.windowTitleWidth = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_Width'] || 816);
    SOUL_MV.ItemBookEx.windowTitleHeight = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_Height'] || 75);
    SOUL_MV.ItemBookEx.windowTitleOpacity = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['WT_OPACITY'] || 255);

    SOUL_MV.ItemBookEx.windowIndexX = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['LIST_X'] || 0);
    SOUL_MV.ItemBookEx.windowIndexY = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['LIST_Y'] || 75);
    SOUL_MV.ItemBookEx.windowIndexWidth = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['LIST_Width'] || 816);
    SOUL_MV.ItemBookEx.windowIndexHeight = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['LIST_Height'] || 146);
    SOUL_MV.ItemBookEx.windowIndexOpacity = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['LIST_OPACITY'] || 255);

    SOUL_MV.ItemBookEx.windowDetailsX = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['D_X'] || 0);
    SOUL_MV.ItemBookEx.windowDetailsY = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['D_Y'] || 220);
    SOUL_MV.ItemBookEx.windowDetailsWidth = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['D_Width'] || 816);
    SOUL_MV.ItemBookEx.windowDetailsHeight = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['D_Height'] || 403);
    SOUL_MV.ItemBookEx.windowDetailsOpacity = Number(PluginManager.parameters('SOUL_MV Item Book Ex')['D_OPACITY'] || 255);


    SOUL_MV.ItemBookEx.titleWindowText = PluginManager.parameters('SOUL_MV Item Book EX')['Window Title'] || 'Item Regalia';
    SOUL_MV.ItemBookEx.unknownData = PluginManager.parameters('SOUL_MV Item Book EX')['Unidentified'] || '??????';
    SOUL_MV.ItemBookEx.itemPriceName = PluginManager.parameters('SOUL_MV Item Book EX')['Item Price'] || 'Price';
    SOUL_MV.ItemBookEx.equipmentText = PluginManager.parameters('SOUL_MV Item Book EX')['Equipment'] || 'Equip';
    SOUL_MV.ItemBookEx.itemTypeText = PluginManager.parameters('SOUL_MV Item Book EX')['Item Type'] || 'Type';

    SOUL_MV.ItemBookEx._command_addOriginalCommands = Window_MenuCommand.prototype.addOriginalCommands;
    Window_MenuCommand.prototype.addOriginalCommands = function() {
        if($gameSystem.showItemBookOnMenu) {
            this.addCommand(SOUL_MV.ItemBookEx.menuName, 'itemBook', true);
        }
        SOUL_MV.ItemBookEx._command_addOriginalCommands.call(this);
    };

    SOUL_MV.ItemBookEx._command_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
    Scene_Menu.prototype.createCommandWindow = function() {
        SOUL_MV.ItemBookEx._command_createCommandWindow.call(this);
        if($gameSystem.showItemBookOnMenu) {
            this._commandWindow.setHandler('itemBook',      this.commandItemBook.bind(this));
        }
    };  

    Scene_Menu.prototype.commandItemBook = function() {
        SceneManager.push(Scene_ItemBookEx);
    };        

    SOUL_MV.ItemBookEx._gameSystem_initialize = Game_System.prototype.initialize;
    Game_System.prototype.initialize = function() {
        SOUL_MV.ItemBookEx._gameSystem_initialize.call(this);
        this._backgroundSprite = SOUL_MV.ItemBookEx.background;
        this.showItemBookOnMenu = SOUL_MV.ItemBookEx.show;
    }

    SOUL_MV.ItemBookEx._interpreterPluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        SOUL_MV.ItemBookEx._interpreterPluginCommand.call(this, command, args);
        if (command === 'ItemBookEX') {
            switch (args[0]) {
            case 'open':
                SceneManager.push(Scene_ItemBookEx);
                break;
            case 'add':
                $gameSystem.addToItemBook(args[1], Number(args[2]));
                break;
            case 'remove':
                $gameSystem.removeFromItemBook(args[1], Number(args[2]));
                break;
            case 'complete':
                $gameSystem.completeItemBook();
                break;
            case 'clear':
                $gameSystem.clearItemBook();
                break;
            case 'show_on_menu':
                if (args[1] === 'true') {
                    $gameSystem.showItemBookOnMenu = true;
                } else if(args[1] === 'false') {
                    $gameSystem.showItemBookOnMenu = false;
                    console.log($gameSystem.showItemBookOnMenu);
                } else {
                    alert("SOUL_MV Item Book EX Error - No such command!");
                }
                break;                
            case 'background':
                $gameSystem._backgroundSprite = args[1];
                break;                
            }
        }
    };

    function Window_ItemBookExTagName() {
        this.initialize.apply(this, arguments);
    }

    Window_ItemBookExTagName.prototype = Object.create(Window_Base.prototype);
    Window_ItemBookExTagName.prototype.constructor = Window_ItemBookExTagName;

    Window_ItemBookExTagName.prototype.initialize = function(x, y) {
        var width = this.windowWidth();
        var height = this.windowHeight();
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this.refresh();
    };

    Window_ItemBookExTagName.prototype.windowWidth = function() {
        return SOUL_MV.ItemBookEx.windowTitleWidth;
    };

    Window_ItemBookExTagName.prototype.windowHeight = function() {
        return SOUL_MV.ItemBookEx.windowTitleHeight;
    };

    Window_ItemBookExTagName.prototype.refresh = function() {
        var x = this.textPadding();
        var width = this.contents.width - this.textPadding() * 2;
        this.contents.clear();
        this.drawTextEx(SOUL_MV.ItemBookEx.titleWindowText, SOUL_MV.ItemBookEx.windowTitleTextX, SOUL_MV.ItemBookEx.windowTitleTextY);
    };

    Window_ItemBookExTagName.prototype.open = function() {
        this.refresh();
        Window_Base.prototype.open.call(this);
    };

    Game_System.prototype.addToItemBook = function(type, dataId) {
        if (!this._ItemBookFlags) {
            this.clearItemBook();
        }
        var typeIndex = this.itemBookTypeToIndex(type);
        if (typeIndex >= 0) {
            this._ItemBookFlags[typeIndex][dataId] = true;
        }
    };

    Game_System.prototype.removeFromItemBook = function(type, dataId) {
        if (this._ItemBookFlags) {
            var typeIndex = this.itemBookTypeToIndex(type);
            if (typeIndex >= 0) {
                this._ItemBookFlags[typeIndex][dataId] = false;
            }
        }
    };

    Game_System.prototype.itemBookTypeToIndex = function(type) {
        switch (type) {
        case 'item':
            return 0;
        case 'weapon':
            return 1;
        case 'armor':
            return 2;
        default:
            return -1;
        }
    };

    Game_System.prototype.completeItemBook = function() {
        var i;
        this.clearItemBook();
        for (i = 1; i < $dataItems.length; i++) {
            this._ItemBookFlags[0][i] = true;
        }
        for (i = 1; i < $dataWeapons.length; i++) {
            this._ItemBookFlags[1][i] = true;
        }
        for (i = 1; i < $dataArmors.length; i++) {
            this._ItemBookFlags[2][i] = true;
        }
    };

    Game_System.prototype.clearItemBook = function() {
        this._ItemBookFlags = [[], [], []];
    };

    Game_System.prototype.isInItemBook = function(item) {
        if (this._ItemBookFlags && item) {
            var typeIndex = -1;
            if (DataManager.isItem(item)) {
                typeIndex = 0;
            } else if (DataManager.isWeapon(item)) {
                typeIndex = 1;
            } else if (DataManager.isArmor(item)) {
                typeIndex = 2;
            }
            if (typeIndex >= 0) {
                return !!this._ItemBookFlags[typeIndex][item.id];
            } else {
                return false;
            }
        } else {
            return false;
        }
    };

    SOUL_MV.ItemBookEx._party_gainItem = Game_Party.prototype.gainItem;
    Game_Party.prototype.gainItem = function(item, amount, includeEquip) {
        SOUL_MV.ItemBookEx._party_gainItem.call(this, item, amount, includeEquip);
        if (item && amount > 0) {
            var type;
            if (DataManager.isItem(item)) {
                type = 'item';
            } else if (DataManager.isWeapon(item)) {
                type = 'weapon';
            } else if (DataManager.isArmor(item)) {
                type = 'armor';
            }
            $gameSystem.addToItemBook(type, item.id);
        }
    };

    function Scene_ItemBookEx() {
        this.initialize.apply(this, arguments);
    }

    Scene_ItemBookEx.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_ItemBookEx.prototype.constructor = Scene_ItemBookEx;

    Scene_ItemBookEx.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_ItemBookEx.prototype.createBackground = function() {
        this._backgroundSprite = new TilingSprite();
        this._backgroundSprite.move(0, 0, Graphics.width, Graphics.height);
        this._backgroundSprite.bitmap = ImageManager.loadItemBookEx($gameSystem._backgroundSprite);
        this.addChild(this._backgroundSprite);
    }

    Scene_ItemBookEx.prototype.update = function() {
        Scene_MenuBase.prototype.update.call(this);
        this._backgroundSprite.origin.x = SOUL_MV.ItemBookEx.backgroundScrollX;
        this._backgroundSprite.origin.y = SOUL_MV.ItemBookEx.backgroundScrollY;
    }

    Scene_ItemBookEx.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);

        // Item Book Name
        this._itembookName = new Window_ItemBookExTagName();
        this._itembookName.x = SOUL_MV.ItemBookEx.windowTitleX;
        this._itembookName.y = SOUL_MV.ItemBookEx.windowTitleY;
        this._itembookName.width = SOUL_MV.ItemBookEx.windowTitleWidth;
        this._itembookName.height = SOUL_MV.ItemBookEx.windowTitleHeight;
        this._itembookName.opacity = SOUL_MV.ItemBookEx.windowTitleOpacity;
        
        // we are now going to change the addWindow to addChild, because using the add
        // window will trigger the same viewpoint as the other windows. That is a bad
        // practice.
        this.addChild(this._itembookName);

        // Item List
        this._indexWindow = new Window_ItemBookIndex(0, 50);
        this._indexWindow.width = SOUL_MV.ItemBookEx.windowIndexWidth;
        this._indexWindow.height = SOUL_MV.ItemBookEx.windowIndexHeight;
        this._indexWindow.setHandler('cancel', this.popScene.bind(this));
        this._indexWindow.x = SOUL_MV.ItemBookEx.windowIndexX;
        this._indexWindow.y = SOUL_MV.ItemBookEx.windowIndexY;
        this._indexWindow.opacity = SOUL_MV.ItemBookEx.windowIndexOpacity;

        // Status Window / Contents
        var ww = SOUL_MV.ItemBookEx.windowDetailsWidth;
        var wh = SOUL_MV.ItemBookEx.windowDetailsHeight;
        this._statusWindow = new Window_ItemBookStatus(SOUL_MV.ItemBookEx.windowDetailsX, SOUL_MV.ItemBookEx.windowDetailsY, ww, wh);
        this._statusWindow.opacity = SOUL_MV.ItemBookEx.windowDetailsOpacity;
        this.addChild(this._indexWindow);
        this.addChild(this._statusWindow);
        this._indexWindow.setStatusWindow(this._statusWindow);
    };

    function Window_ItemBookIndex() {
        this.initialize.apply(this, arguments);
    }

    Window_ItemBookIndex.prototype = Object.create(Window_Selectable.prototype);
    Window_ItemBookIndex.prototype.constructor = Window_ItemBookIndex;

    Window_ItemBookIndex.lastTopRow = 0;
    Window_ItemBookIndex.lastIndex  = 0;

    Window_ItemBookIndex.prototype.initialize = function(x, y) {
        var width = 816;
        var height = this.fittingHeight(6);
        Window_Selectable.prototype.initialize.call(this, x, y, width, height);
        this.refresh();
        this.setTopRow(Window_ItemBookIndex.lastTopRow);
        this.select(Window_ItemBookIndex.lastIndex);
        this.activate();
    };

    Window_ItemBookIndex.prototype.maxCols = function() {
        return 3;
    };

    Window_ItemBookIndex.prototype.maxItems = function() {
        return this._list ? this._list.length : 0;
    };

    Window_ItemBookIndex.prototype.setStatusWindow = function(statusWindow) {
        this._statusWindow = statusWindow;
        this.updateStatus();
    };

    Window_ItemBookIndex.prototype.update = function() {
        Window_Selectable.prototype.update.call(this);
        this.updateStatus();
    };

    Window_ItemBookIndex.prototype.updateStatus = function() {
        if (this._statusWindow) {
            var item = this._list[this.index()];
            this._statusWindow.setItem(item);
        }
    };

    Window_ItemBookIndex.prototype.refresh = function() {
        var i, item;
        this._list = [];
        for (i = 1; i < $dataItems.length; i++) {
            item = $dataItems[i];
            if (item.name && item.itypeId === 1 || item.name && item.itypeId === 2) {
                if (!item.note.match(/<item book exclude>/)) {
                    this._list.push(item);
                }
            }
        }
        for (i = 1; i < $dataWeapons.length; i++) {
            item = $dataWeapons[i];
            if (item.name && !item.note.match(/<item book exclude>/)) {
                this._list.push(item);
            }
        }
        for (i = 1; i < $dataArmors.length; i++) {
            item = $dataArmors[i];
            if (item.name && !item.note.match(/<item book exclude>/)) {
                this._list.push(item);
            }
        }
        this.createContents();
        this.drawAllItems();
    };

    Window_ItemBookIndex.prototype.drawItem = function(index) {
        var item = this._list[index];
        var rect = this.itemRect(index);
        var width = rect.width - this.textPadding();
        if ($gameSystem.isInItemBook(item)) {
            this.drawItemName(item, rect.x, rect.y, width);
        } else {
            var iw = Window_Base._iconWidth + 4;
            this.drawText(SOUL_MV.ItemBookEx.unknownData, rect.x + iw, rect.y, width - iw);
        }
    };

    Window_ItemBookIndex.prototype.processCancel = function() {
        Window_Selectable.prototype.processCancel.call(this);
        Window_ItemBookIndex.lastTopRow = this.topRow();
        Window_ItemBookIndex.lastIndex = this.index();
    };

    function Window_ItemBookStatus() {
        this.initialize.apply(this, arguments);
    }

    Window_ItemBookStatus.prototype = Object.create(Window_Base.prototype);
    Window_ItemBookStatus.prototype.constructor = Window_ItemBookStatus;

    Window_ItemBookStatus.prototype.initialize = function(x, y, width, height) {
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this._contentSprite = new Sprite();
        this._contentSprite.x = SOUL_MV.ItemBookEx.pictureX;
        this._contentSprite.y = SOUL_MV.ItemBookEx.pictureY;
        this._contentSprite.opacity = SOUL_MV.ItemBookEx.pictureOpacity;        
        this.addChild(this._contentSprite);
    };

    Window_ItemBookStatus.prototype.setItem = function(item) {
        if (this._item !== item) {
            this._item = item;
            this.refresh();
        }
    };

    ImageManager.loadItemBookEx = function(filename, hue) {
        return this.loadBitmap('img/ItemBookEx/', filename, hue, true);
    };

    Window_ItemBookStatus.prototype.refresh = function() {
        var item = this._item;
        var x = 0;
        var y = 0;
        var lineHeight = this.lineHeight();

        this.contents.clear();
        if (this._item !== null || this._item !== undefined) {
            if ($gameSystem.isInItemBook(this._item)) {
                this._contentSprite.bitmap = ImageManager.loadItemBookEx(this._item.name);
            } else {
                this._contentSprite.bitmap = ImageManager.loadItemBookEx(null);
            }
        } else {
            this._contentSprite.bitmap = null;
        }
        

        if (!item || !$gameSystem.isInItemBook(item)) {
            return;
        }

        this.drawItemName(item, x, y);

        x = this.textPadding();
        y = lineHeight + this.textPadding();

        var price = item.price > 0 ? item.price : '-';
        this.changeTextColor(this.systemColor());
        this.drawText(SOUL_MV.ItemBookEx.itemPriceName, x, y, 120);
        this.resetTextColor();
        this.drawText(price, x + 120, y, 120, 'right');
        y += lineHeight;

        if (DataManager.isWeapon(item) || DataManager.isArmor(item)) {
            var etype = $dataSystem.equipTypes[item.etypeId];
            this.changeTextColor(this.systemColor());
            this.drawText(SOUL_MV.ItemBookEx.equipmentText, x, y, 120);
            this.resetTextColor();
            this.drawText(etype, x + 120, y, 120, 'right');
            y += lineHeight;

            var type;
            if (DataManager.isWeapon(item)) {
                type = $dataSystem.weaponTypes[item.wtypeId];
            } else {
                type = $dataSystem.armorTypes[item.atypeId];
            }
            this.changeTextColor(this.systemColor());
            this.drawText(SOUL_MV.ItemBookEx.itemTypeText, x, y, 120);
            this.resetTextColor();
            this.drawText(type, x + 120, y, 120, 'right');

            x = this.textPadding() + 300;
            y = lineHeight + this.textPadding();
            for (var i = 2; i < 8; i++) {
                this.changeTextColor(this.systemColor());
                this.drawText(TextManager.param(i), x, y, 160);
                this.resetTextColor();
                this.drawText(item.params[i], x + 160, y, 60, 'right');
                y += lineHeight;
            }
        }

        x = 0;
        y = this.textPadding() * 2 + lineHeight * 7;
        this.drawTextEx(item.description, 5, y);
    };

})();
